import { Factory, Initializer, Destructor, FactoryResolver } from './factory';
import { Injection } from './injection';
import { SpecifierResolver } from './specifier';
import { Dict } from './dict';
export default class FactoryRegistry<Specifier, T> implements FactoryResolver<Specifier, T> {
    private _specifierResolver;
    private _factories;
    private _initializers;
    private _destructors;
    private _constructorArguments;
    private _initializerArguments;
    private _destructorArguments;
    constructor(specifierResolver: SpecifierResolver<Specifier>);
    registerFactory(specifier: Specifier, factory: Factory<T>): void;
    unregisterFactory(specifier: Specifier): void;
    factoryFor(specifier: Specifier): Factory<T>;
    registerInitializer(specifier: Specifier, initializer: Initializer<T>): void;
    unregisterInitializer(specifier: Specifier, name: string): void;
    unregisterInitializers(specifier: Specifier): void;
    initializersFor(specifier: Specifier): Initializer<T>[];
    registerDestructor(specifier: Specifier, destructor: Destructor<T>): void;
    unregisterDestructor(specifier: Specifier, name: string): void;
    unregisterDestructors(specifier: Specifier): void;
    destructorsFor(specifier: Specifier): Destructor<T>[];
    registerConstructorArguments(specifier: Specifier, args: Injection<Specifier>[]): void;
    unregisterConstructorArguments(specifier: Specifier): void;
    constructorArgumentsFor(specifier: Specifier): Injection<Specifier>[];
    registerInitializerArguments(specifier: Specifier, name: string, args: Injection<Specifier>[]): void;
    unregisterInitializerArguments(specifier: Specifier, name?: string): void;
    initializerArgumentsFor(specifier: Specifier, name: string): Injection<Specifier>[];
    registerDestructorArguments(specifier: Specifier, name: string, args: Injection<Specifier>[]): void;
    unregisterDestructorArguments(specifier: Specifier, name?: string): void;
    destructorArgumentsFor(specifier: Specifier, name: string): Injection<Specifier>[];
    retrieveFactory(specifier: Specifier): Factory<T>;
    retrieveInitializers(specifier: Specifier): Dict<Initializer<T>>;
    retrieveDestructors(specifier: Specifier): Dict<Destructor<T>>;
    retrieveConstructorArguments(specifier: Specifier): Injection<Specifier>[];
    retrieveInitializerArguments(specifier: Specifier, name: string): Injection<Specifier>[];
    retrieveDestructorArguments(specifier: Specifier, name: string): Injection<Specifier>[];
}
